@@ -1,4 +1,5 @@
 #include "recording.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <glib.h>
@@ -13,8 +14,6 @@
 #include <assert.h>
 #include <stdarg.h>
 
-#include "xt_RTPENGINE.h"
-
 #include "call.h"
 #include "main.h"
 #include "kernel.h"
@@ -23,8 +22,9 @@
 #include "cdr.h"
 #include "log.h"
 #include "call_interfaces.h"
+#include "media_player.h"
 
-
+#include "xt_RTPENGINE.h"
 
 struct rec_pcap_format {
 	int linktype;
@@ -35,35 +35,37 @@ struct rec_pcap_format {
 
 
 static int check_main_spool_dir(const char *spoolpath);
-static char *recording_setup_file(struct recording *recording);
-static char *meta_setup_file(struct recording *recording);
+static char *recording_setup_file(struct recording *recording, const str *);
+static char *meta_setup_file(struct recording *recording, const str *);
 static int append_meta_chunk(struct recording *recording, const char *buf, unsigned int buflen,
 		const char *label_fmt, ...)
 	__attribute__((format(printf,4,5)));
+static int vappend_meta_chunk(struct recording *recording, const char *buf, unsigned int buflen,
+		const char *label_fmt, va_list ap);
 
 // all methods
 static int create_spool_dir_all(const char *spoolpath);
-static void init_all(struct call *call);
+static void init_all(call_t *call);
 static void sdp_after_all(struct recording *recording, GString *str, struct call_monologue *ml,
 		enum call_opmode opmode);
 static void dump_packet_all(struct media_packet *mp, const str *s);
-static void finish_all(struct call *call, bool discard);
+static void finish_all(call_t *call, bool discard);
 
 // pcap methods
 static int rec_pcap_create_spool_dir(const char *dirpath);
-static void rec_pcap_init(struct call *);
+static void rec_pcap_init(call_t *);
 static void sdp_after_pcap(struct recording *, GString *str, struct call_monologue *, enum call_opmode opmode);
 static void dump_packet_pcap(struct media_packet *mp, const str *s);
-static void finish_pcap(struct call *, bool discard);
+static void finish_pcap(call_t *, bool discard);
 static void response_pcap(struct recording *, bencode_item_t *);
 
 // proc methods
-static void proc_init(struct call *);
+static void proc_init(call_t *);
 static void sdp_before_proc(struct recording *, const str *, struct call_monologue *, enum call_opmode);
 static void sdp_after_proc(struct recording *, GString *str, struct call_monologue *, enum call_opmode opmode);
 static void meta_chunk_proc(struct recording *, const char *, const str *);
-static void update_flags_proc(struct call *call, bool streams);
-static void finish_proc(struct call *, bool discard);
+static void update_flags_proc(call_t *call, bool streams);
+static void finish_proc(call_t *, bool discard);
 static void dump_packet_proc(struct media_packet *mp, const str *s);
 static void init_stream_proc(struct packet_stream *);
 static void setup_stream_proc(struct packet_stream *);
@@ -74,6 +76,7 @@ static void kernel_info_proc(struct packet_stream *, struct rtpengine_target_inf
 static void rec_pcap_eth_header(unsigned char *, struct packet_stream *);
 
 #define append_meta_chunk_str(r, str, f...) append_meta_chunk(r, (str)->s, (str)->len, f)
+#define vappend_meta_chunk_str(r, str, f, ap) vappend_meta_chunk(r, (str)->s, (str)->len, f, ap)
 #define append_meta_chunk_s(r, str, f...) append_meta_chunk(r, (str), strlen(str), f)
 #define append_meta_chunk_null(r,f...) append_meta_chunk(r, "", 0, f)
 
@@ -269,60 +272,86 @@ static int rec_pcap_create_spool_dir(const char *spoolpath) {
 }
 
 // lock must be held
-void update_metadata_call(struct call *call, str *metadata) {
-	if (!metadata || !metadata->s || !call)
+static void update_call_field(call_t *call, str *dst_field, const str *src_field, const char *meta_fmt, ...) {
+	if (!call)
 		return;
 
-	if (str_cmp_str(metadata, &call->metadata)) {
-		call_str_cpy(call, &call->metadata, metadata);
-		if (call->recording)
-			recording_meta_chunk(call->recording, "METADATA", metadata);
+	if (src_field && src_field->len && str_cmp_str(src_field, dst_field))
+		call_str_cpy(call, dst_field, src_field);
+
+	if (call->recording && dst_field->len) {
+		va_list ap;
+		va_start(ap, meta_fmt);
+		vappend_meta_chunk_str(call->recording, dst_field, meta_fmt, ap);
+		va_end(ap);
 	}
 }
 
 // lock must be held
-void update_metadata_monologue(struct call_monologue *ml, str *metadata) {
-	if (!metadata || !metadata->s || !ml)
-		return;
-
-	struct call *call = ml->call;
-
-	if (str_cmp_str(metadata, &ml->metadata)) {
-		call_str_cpy(call, &ml->metadata, metadata);
-		if (call->recording)
-			append_meta_chunk_str(call->recording, metadata, "METADATA-TAG %u", ml->unique_id);
+void update_metadata_call(call_t *call, const sdp_ng_flags *flags) {
+	if (flags && flags->skip_recording_db)
+		CALL_SET(call, NO_REC_DB);
+	if (call->recording) {
+		// must come first because METADATA triggers update to DB
+		if (CALL_ISSET(call, NO_REC_DB))
+			append_meta_chunk_null(call->recording, "SKIP_DATABASE");
 	}
 
-	update_metadata_call(call, metadata);
-}
-
-static void update_output_dest(struct call *call, const str *output_dest) {
-	if (!output_dest || !output_dest->s || !call->recording)
-		return;
-	recording_meta_chunk(call->recording, "OUTPUT_DESTINATION", output_dest);
+	update_call_field(call, &call->metadata, flags ? &flags->metadata : NULL, "METADATA");
+	update_call_field(call, &call->recording_file, flags ? &flags->recording_file : NULL, "RECORDING_FILE");
+	update_call_field(call, &call->recording_path, flags ? &flags->recording_path : NULL, "RECORDING_PATH");
+	update_call_field(call, &call->recording_pattern, flags ? &flags->recording_pattern : NULL,
+			"RECORDING_PATTERN");
 }
 
 // lock must be held
-static void update_flags_proc(struct call *call, bool streams) {
+void update_metadata_monologue_only(struct call_monologue *ml, const sdp_ng_flags *flags) {
+	if (!ml)
+		return;
+
+	update_call_field(ml->call, &ml->metadata, flags ? &flags->metadata : NULL,
+			"METADATA-TAG %u", ml->unique_id);
+	update_call_field(ml->call, &ml->label, NULL, "LABEL %u", ml->unique_id);
+}
+
+void update_metadata_monologue(struct call_monologue *ml, const sdp_ng_flags *flags) {
+	if (!ml)
+		return;
+
+	update_metadata_monologue_only(ml, flags);
+	update_metadata_call(ml->call, flags);
+}
+
+// lock must be held
+static void update_flags_proc(call_t *call, bool streams) {
 	append_meta_chunk_null(call->recording, "RECORDING %u", CALL_ISSET(call, RECORDING_ON));
 	append_meta_chunk_null(call->recording, "FORWARDING %u", CALL_ISSET(call, REC_FORWARDING));
+	update_metadata_call(call, NULL);
 	if (!streams)
 		return;
-	for (GList *l = call->streams.head; l; l = l->next) {
+	for (__auto_type l = call->streams.head; l; l = l->next) {
 		struct packet_stream *ps = l->data;
 		append_meta_chunk_null(call->recording, "STREAM %u FORWARDING %u",
 				ps->unique_id, ML_ISSET(ps->media->monologue, REC_FORWARDING) ? 1 : 0);
 	}
 }
-static void recording_update_flags(struct call *call, bool streams) {
+static void recording_update_flags(call_t *call, bool streams) {
 	_rm(update_flags, call, streams);
 }
 
+static void rec_setup_monologue(struct call_monologue *ml) {
+	recording_setup_monologue(ml);
+	if (ml->rec_player) {
+		bool ret = media_player_start(ml->rec_player);
+		if (!ret)
+			ilog(LOG_WARN, "Failed to start media player for recording announcement");
+	}
+}
+
 // lock must be held
-void recording_start(struct call *call, const char *prefix, const str *output_dest) {
+void recording_start_daemon(call_t *call) {
 	if (call->recording) {
 		// already active
-		update_output_dest(call, output_dest);
 		recording_update_flags(call, true);
 		return;
 	}
@@ -334,39 +363,36 @@ void recording_start(struct call *call, const char *prefix, const str *output_de
 	ilog(LOG_NOTICE, "Turning on call recording.");
 
 	call->recording = g_slice_alloc0(sizeof(struct recording));
-	struct recording *recording = call->recording;
-	recording->escaped_callid = g_uri_escape_string(call->callid.s, NULL, 0);
-	if (!prefix) {
+	g_autoptr(char) escaped_callid = g_uri_escape_string(call->callid.s, NULL, 0);
+	if (!call->recording_meta_prefix.len) {
 		const int rand_bytes = 8;
 		char rand_str[rand_bytes * 2 + 1];
 		rand_hex_str(rand_str, rand_bytes);
-		recording->meta_prefix = g_strdup_printf("%s-%s", recording->escaped_callid, rand_str);
+		g_autoptr(char) meta_prefix = g_strdup_printf("%s-%s", escaped_callid, rand_str);
+		call_str_cpy(call, &call->recording_meta_prefix, &STR_INIT(meta_prefix));
+		call_str_cpy(call, &call->recording_random_tag, &STR_CONST_INIT(rand_str));
 	}
-	else
-		recording->meta_prefix = g_strdup(prefix);
 
 	_rm(init_struct, call);
 
 	// update main call flags (global recording/forwarding on/off) to prevent recording
 	// features from being started when the stream info (through setup_stream) is
 	// propagated if recording is actually off
-	update_output_dest(call, output_dest);
 	recording_update_flags(call, false);
 
 	// if recording has been turned on after initial call setup, we must walk
 	// through all related objects and initialize the recording stuff. if this
 	// function is called right at the start of the call, all of the following
 	// is essentially a no-op
-	GList *l;
-	for (l = call->monologues.head; l; l = l->next) {
+	for (__auto_type l = call->monologues.head; l; l = l->next) {
 		struct call_monologue *ml = l->data;
-		recording_setup_monologue(ml);
+		rec_setup_monologue(ml);
 	}
-	for (l = call->medias.head; l; l = l->next) {
+	for (__auto_type l = call->medias.head; l; l = l->next) {
 		struct call_media *m = l->data;
 		recording_setup_media(m);
 	}
-	for (l = call->streams.head; l; l = l->next) {
+	for (__auto_type l = call->streams.head; l; l = l->next) {
 		struct packet_stream *ps = l->data;
 		recording_setup_stream(ps);
 		__unkernelize(ps, "recording start");
@@ -375,7 +401,13 @@ void recording_start(struct call *call, const char *prefix, const str *output_de
 
 	recording_update_flags(call, true);
 }
-void recording_stop(struct call *call) {
+// lock must be held
+void recording_start(call_t *call) {
+	CALL_SET(call, RECORDING_ON);
+	recording_start_daemon(call);
+}
+// lock must be held
+void recording_stop_daemon(call_t *call) {
 	if (!call->recording)
 		return;
 
@@ -385,7 +417,7 @@ void recording_stop(struct call *call) {
 		return;
 	}
 
-	for (GList *l = call->monologues.head; l; l = l->next) {
+	for (__auto_type l = call->monologues.head; l; l = l->next) {
 		struct call_monologue *ml = l->data;
 		if (ML_ISSET(ml, REC_FORWARDING)) {
 			recording_update_flags(call, true);
@@ -396,11 +428,21 @@ void recording_stop(struct call *call) {
 	ilog(LOG_NOTICE, "Turning off call recording.");
 	recording_finish(call, false);
 }
-void recording_pause(struct call *call) {
+// lock must be held
+void recording_stop(call_t *call) {
+	CALL_CLEAR(call, RECORDING_ON);
+	recording_stop_daemon(call);
+}
+// lock must be held
+void recording_pause(call_t *call) {
+	CALL_CLEAR(call, RECORDING_ON);
+	if (!call->recording)
+		return;
 	ilog(LOG_NOTICE, "Pausing call recording.");
 	recording_update_flags(call, true);
 }
-void recording_discard(struct call *call) {
+// lock must be held
+void recording_discard(call_t *call) {
 	CALL_CLEAR(call, RECORDING_ON);
 	if (!call->recording)
 		return;
@@ -411,7 +453,7 @@ void recording_discard(struct call *call) {
 
 /**
  *
- * Controls the setting of recording variables on a `struct call *`.
+ * Controls the setting of recording variables on a `call_t *`.
  * Sets the `record_call` value on the `struct call`, initializing the
  * recording struct if necessary.
  * If we do not yet have a PCAP file associated with the call, create it
@@ -419,38 +461,32 @@ void recording_discard(struct call *call) {
  *
  * Returns a boolean for whether or not the call is being recorded.
  */
-void detect_setup_recording(struct call *call, const struct sdp_ng_flags *flags) {
+void detect_setup_recording(call_t *call, const sdp_ng_flags *flags) {
 	if (!flags)
 		return;
 
 	const str *recordcall = &flags->record_call_str;
 
-	if (!str_cmp(recordcall, "yes") || !str_cmp(recordcall, "on") || flags->record_call) {
-		CALL_SET(call, RECORDING_ON);
-		recording_start(call, NULL, &flags->output_dest);
-	}
-	else if (!str_cmp(recordcall, "no") || !str_cmp(recordcall, "off")) {
-		CALL_CLEAR(call, RECORDING_ON);
+	if (!str_cmp(recordcall, "yes") || !str_cmp(recordcall, "on") || flags->record_call)
+		recording_start(call);
+	else if (!str_cmp(recordcall, "no") || !str_cmp(recordcall, "off"))
 		recording_stop(call);
-	}
-	else if (!str_cmp(recordcall, "discard") || flags->discard_recording) {
-		CALL_CLEAR(call, RECORDING_ON);
+	else if (!str_cmp(recordcall, "discard") || flags->discard_recording)
 		recording_discard(call);
-	}
 	else if (recordcall->len != 0)
 		ilog(LOG_INFO, "\"record-call\" flag "STR_FORMAT" is invalid flag.", STR_FMT(recordcall));
 }
 
-static void rec_pcap_init(struct call *call) {
+static void rec_pcap_init(call_t *call) {
 	struct recording *recording = call->recording;
 
 	// Wireshark starts at packet index 1, so we start there, too
 	recording->pcap.packet_num = 1;
 	mutex_init(&recording->pcap.recording_lock);
-	meta_setup_file(recording);
+	meta_setup_file(recording, &call->recording_meta_prefix);
 
 	// set up pcap file
-	char *pcap_path = recording_setup_file(recording);
+	char *pcap_path = recording_setup_file(recording, &call->recording_meta_prefix);
 	if (pcap_path != NULL && recording->pcap.recording_pdumper != NULL
 	    && recording->pcap.meta_fp) {
 		// Write the location of the PCAP file to the metadata file
@@ -466,13 +502,13 @@ static char *file_path_str(const char *id, const char *prefix, const char *suffi
  * Create a call metadata file in a temporary location.
  * Attaches the filepath and the file pointer to the call struct.
  */
-static char *meta_setup_file(struct recording *recording) {
+static char *meta_setup_file(struct recording *recording, const str *meta_prefix) {
 	if (spooldir == NULL) {
 		// No spool directory was created, so we cannot have metadata files.
 		return NULL;
 	}
 
-	char *meta_filepath = file_path_str(recording->meta_prefix, "/tmp/rtpengine-meta-", ".tmp");
+	char *meta_filepath = file_path_str(meta_prefix->s, "/tmp/rtpengine-meta-", ".tmp");
 	recording->pcap.meta_filepath = meta_filepath;
 	FILE *mfp = fopen(meta_filepath, "w");
 	// coverity[check_return : FALSE]
@@ -517,7 +553,7 @@ static void sdp_after_pcap(struct recording *recording, GString *s, struct call_
 /**
  * Writes metadata to metafile, closes file, and renames it to finished location.
  */
-static void rec_pcap_meta_finish_file(struct call *call) {
+static void rec_pcap_meta_finish_file(call_t *call) {
 	// This should usually be called from a place that has the call->master_lock
 	struct recording *recording = call->recording;
 
@@ -590,7 +626,7 @@ static void rec_pcap_meta_finish_file(struct call *call) {
 /**
  * Closes and discards all output files.
  */
-static void rec_pcap_meta_discard_file(struct call *call) {
+static void rec_pcap_meta_discard_file(call_t *call) {
 	struct recording *recording = call->recording;
 
 	if (recording == NULL || recording->pcap.meta_fp == NULL)
@@ -608,7 +644,7 @@ static void rec_pcap_meta_discard_file(struct call *call) {
  * Generate a random PCAP filepath to write recorded RTP stream.
  * Returns path to created file.
  */
-static char *recording_setup_file(struct recording *recording) {
+static char *recording_setup_file(struct recording *recording, const str *meta_prefix) {
 	char *recording_path = NULL;
 
 	if (!spooldir)
@@ -616,7 +652,7 @@ static char *recording_setup_file(struct recording *recording) {
 	if (recording->pcap.recording_pd || recording->pcap.recording_pdumper)
 		return NULL;
 
-	recording_path = file_path_str(recording->meta_prefix, "/pcaps/", ".pcap");
+	recording_path = file_path_str(meta_prefix->s, "/pcaps/", ".pcap");
 	recording->pcap.recording_path = recording_path;
 
 	recording->pcap.recording_pd = pcap_open_dead(rec_pcap_format->linktype, 65535);
@@ -711,7 +747,7 @@ static void dump_packet_pcap(struct media_packet *mp, const str *s) {
 	mutex_unlock(&recording->pcap.recording_lock);
 }
 
-static void finish_pcap(struct call *call, bool discard) {
+static void finish_pcap(call_t *call, bool discard) {
 	rec_pcap_recording_finish_file(call->recording);
 	if (!discard)
 		rec_pcap_meta_finish_file(call);
@@ -733,7 +769,7 @@ static void response_pcap(struct recording *recording, bencode_item_t *output) {
 
 
 
-void recording_finish(struct call *call, bool discard) {
+void recording_finish(call_t *call, bool discard) {
 	if (!call || !call->recording)
 		return;
 
@@ -743,9 +779,6 @@ void recording_finish(struct call *call, bool discard) {
 
 	_rm(finish, call, discard);
 
-	g_clear_pointer(&recording->meta_prefix, g_free);
-	g_clear_pointer(&recording->escaped_callid, free);
-
 	g_slice_free1(sizeof(*(recording)), recording);
 	call->recording = NULL;
 }
@@ -798,22 +831,30 @@ static int vappend_meta_chunk_iov(struct recording *recording, struct iovec *in_
 	return 0;
 }
 
-static int append_meta_chunk(struct recording *recording, const char *buf, unsigned int buflen,
-		const char *label_fmt, ...)
+static int vappend_meta_chunk(struct recording *recording, const char *buf, unsigned int buflen,
+		const char *label_fmt, va_list ap)
 {
 	struct iovec iov;
 	iov.iov_base = (void *) buf;
 	iov.iov_len = buflen;
 
+	int ret = vappend_meta_chunk_iov(recording, &iov, 1, buflen, label_fmt, ap);
+
+	return ret;
+}
+
+static int append_meta_chunk(struct recording *recording, const char *buf, unsigned int buflen,
+		const char *label_fmt, ...)
+{
 	va_list ap;
 	va_start(ap, label_fmt);
-	int ret = vappend_meta_chunk_iov(recording, &iov, 1, buflen, label_fmt, ap);
+	int ret = vappend_meta_chunk(recording, buf, buflen, label_fmt, ap);
 	va_end(ap);
 
 	return ret;
 }
 
-static void proc_init(struct call *call) {
+static void proc_init(call_t *call) {
 	struct recording *recording = call->recording;
 
 	recording->proc.call_idx = UNINIT_IDX;
@@ -821,20 +862,19 @@ static void proc_init(struct call *call) {
 		ilog(LOG_WARN, "Call recording through /proc interface requested, but kernel table not open");
 		return;
 	}
-	recording->proc.call_idx = kernel_add_call(recording->meta_prefix);
+	recording->proc.call_idx = kernel_add_call(call->recording_meta_prefix.s);
 	if (recording->proc.call_idx == UNINIT_IDX) {
 		ilog(LOG_ERR, "Failed to add call to kernel recording interface: %s", strerror(errno));
 		return;
 	}
 	ilog(LOG_DEBUG, "kernel call idx is %u", recording->proc.call_idx);
 
-	recording->proc.meta_filepath = file_path_str(recording->meta_prefix, "/", ".meta");
+	recording->proc.meta_filepath = file_path_str(call->recording_meta_prefix.s, "/", ".meta");
 	unlink(recording->proc.meta_filepath); // start fresh XXX good idea?
 
 	append_meta_chunk_str(recording, &call->callid, "CALL-ID");
-	append_meta_chunk_s(recording, recording->meta_prefix, "PARENT");
-	if (call->metadata.len)
-		recording_meta_chunk(recording, "METADATA", &call->metadata);
+	append_meta_chunk_s(recording, call->recording_meta_prefix.s, "PARENT");
+	append_meta_chunk_s(recording, call->recording_random_tag.s, "RANDOM_TAG");
 }
 
 static void sdp_before_proc(struct recording *recording, const str *sdp, struct call_monologue *ml,
@@ -851,7 +891,7 @@ static void sdp_after_proc(struct recording *recording, GString *s, struct call_
 			"SDP from %u after %s", ml->unique_id, get_opmode_text(opmode));
 }
 
-static void finish_proc(struct call *call, bool discard) {
+static void finish_proc(call_t *call, bool discard) {
 	struct recording *recording = call->recording;
 	if (!kernel.is_open)
 		return;
@@ -859,13 +899,13 @@ static void finish_proc(struct call *call, bool discard) {
 		kernel_del_call(recording->proc.call_idx);
 		recording->proc.call_idx = UNINIT_IDX;
 	}
-	for (GList *l = call->streams.head; l; l = l->next) {
+	for (__auto_type l = call->streams.head; l; l = l->next) {
 		struct packet_stream *ps = l->data;
 		ps->recording.proc.stream_idx = UNINIT_IDX;
 	}
 
 	const char *unlink_fn = recording->proc.meta_filepath;
-	AUTO_CLEANUP_GBUF(discard_fn);
+	g_autoptr(char) discard_fn = NULL;
 	if (discard) {
 		discard_fn = g_strdup_printf("%s.DISCARD", recording->proc.meta_filepath);
 		int ret = rename(recording->proc.meta_filepath, discard_fn);
@@ -892,7 +932,7 @@ static void init_stream_proc(struct packet_stream *stream) {
 static void setup_stream_proc(struct packet_stream *stream) {
 	struct call_media *media = stream->media;
 	struct call_monologue *ml = media->monologue;
-	struct call *call = stream->call;
+	call_t *call = stream->call;
 	struct recording *recording = call->recording;
 	char buf[128];
 	int len;
@@ -906,9 +946,9 @@ static void setup_stream_proc(struct packet_stream *stream) {
 	if (ML_ISSET(ml, NO_RECORDING))
 		return;
 
-	len = snprintf(buf, sizeof(buf), "TAG %u MEDIA %u TAG-MEDIA %u COMPONENT %u FLAGS %u",
-			ml->unique_id, media->unique_id, media->index, stream->component,
-			stream->ps_flags);
+	len = snprintf(buf, sizeof(buf), "TAG %u MEDIA %u TAG-MEDIA %u COMPONENT %u FLAGS %" PRIu64 " MEDIA-SDP-ID %i",
+				   ml->unique_id, media->unique_id, media->index, stream->component,
+				   atomic64_get_na(&stream->ps_flags), media->media_sdp_id);
 	append_meta_chunk(recording, buf, len, "STREAM %u details", stream->unique_id);
 
 	len = snprintf(buf, sizeof(buf), "tag-%u-media-%u-component-%u-%s-id-%u",
@@ -925,7 +965,7 @@ static void setup_stream_proc(struct packet_stream *stream) {
 }
 
 static void setup_monologue_proc(struct call_monologue *ml) {
-	struct call *call = ml->call;
+	call_t *call = ml->call;
 	struct recording *recording = call->recording;
 
 	if (!recording)
@@ -934,14 +974,11 @@ static void setup_monologue_proc(struct call_monologue *ml) {
 		return;
 
 	append_meta_chunk_str(recording, &ml->tag, "TAG %u", ml->unique_id);
-	if (ml->label.len)
-		append_meta_chunk_str(recording, &ml->label, "LABEL %u", ml->unique_id);
-	if (ml->metadata.len)
-		append_meta_chunk_str(recording, &ml->metadata, "METADATA-TAG %u", ml->unique_id);
+	update_metadata_monologue_only(ml, NULL);
 }
 
 static void setup_media_proc(struct call_media *media) {
-	struct call *call = media->call;
+	call_t *call = media->call;
 	struct recording *recording = call->recording;
 
 	if (!recording)
@@ -951,17 +988,16 @@ static void setup_media_proc(struct call_media *media) {
 
 	append_meta_chunk_null(recording, "MEDIA %u PTIME %i", media->unique_id, media->ptime);
 
-	GList *pltypes = g_hash_table_get_values(media->codecs.codecs);
+	codecs_ht_iter iter;
+	t_hash_table_iter_init(&iter, media->codecs.codecs);
 
-	for (GList *l = pltypes; l; l = l->next) {
-		struct rtp_payload_type *pt = l->data;
+	rtp_payload_type *pt;
+	while (t_hash_table_iter_next(&iter, NULL, &pt)) {
 		append_meta_chunk(recording, pt->encoding_with_params.s, pt->encoding_with_params.len,
 				"MEDIA %u PAYLOAD TYPE %u", media->unique_id, pt->payload_type);
 		append_meta_chunk(recording, pt->format_parameters.s, pt->format_parameters.len,
 				"MEDIA %u FMTP %u", media->unique_id, pt->payload_type);
 	}
-
-	g_list_free(pltypes);
 }
 
 
@@ -1014,7 +1050,7 @@ static int create_spool_dir_all(const char *spoolpath) {
 	return TRUE;
 }
 
-static void init_all(struct call *call) {
+static void init_all(call_t *call) {
 	rec_pcap_init(call);
 	proc_init(call);
 }
@@ -1031,7 +1067,7 @@ static void dump_packet_all(struct media_packet *mp, const str *s) {
 	dump_packet_proc(mp, s);
 }
 
-static void finish_all(struct call *call, bool discard) {
+static void finish_all(call_t *call, bool discard) {
 	finish_pcap(call, discard);
 	finish_proc(call, discard);
 }
